#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from re import findall as re_findall
from urllib.request import urlopen

from pymarketcap import Pymarketcap
pym = Pymarketcap()

class TypeTester:
    def _name(self, value): assert type(value) == str
    def _symbol(self, value): assert type(value) == str
    def _platform(self, value): assert type(value) in [str, type(None)]
    def _market_cap(self, value): assert type(value) in [float, type(None)]
    def _price(self, value): assert type(value) in [float, type(None)]
    def _circulating_supply(self, value): assert type(value) in [float, type(None)]
    def _volume_24h(self, value): assert type(value) in [float, type(None)]

tt = TypeTester()

def assert_types(res):
    for currency in res:
        for key, value in currency.items():
            eval("tt._{}({})".format(
                key,  # Strings need to be "quoted":
                value if type(value) != str else '"%s"' % value
            ))

def test_types():
    res = pym.tokens()
    assert_types(res)

def assert_number_of_tokens(res):
    req = urlopen("https://coinmarketcap.com/tokens/views/all/")
    data = req.read()
    req.close()
    indexes = int(
        re_findall(r'<td class="text-center">\n(\d+)\n</td>', data.decode())[-1]
    )
    assert len(res) in [indexes-1, indexes, indexes+1]

def test_consistence():
    res = pym.tokens()
    assert_number_of_tokens(res)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import re
from json import loads

from precompiler.utils import GET

def get_badges_list():
    """Get list of convertible badges from coinmarketcap documentation"""
    url = "https://coinmarketcap.com/api/"
    res = GET(url).decode()
    raw_badges = re.search(r"Valid values are: <br>\s*(.*)\s*</li>", res)
    response = loads("[%s]" % raw_badges.group(1))
    response.append("USD")
    return sorted(response)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import asyncio
from pymarketcap import AsyncPymarketcap

async def main():
    async with AsyncPymarketcap() as apym:
        async for currency in apym.every_currency():
            print(currency)

if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import choice

import pytest

from pymarketcap.tests.currency import (
    assert_types,
    assert_consistence
)
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def test_types():
    coin = choice(pym.coins)
    print("(Currency: %s)" % coin, end=" ")
    res = pym.currency(coin)
    assert_types(res)

def test_consistence():
    coin = choice(pym.coins)
    print("(Currency: %s)" % coin, end=" ")
    res = pym.currency(coin)
    assert_consistence(res)

def test_convert():
    coin = choice(pym.coins)
    print("(Currency: %s)" % coin, end=" ")
    res = pym.currency(coin, convert="BTC")

    assert_types(res)
    assert_consistence(res)

def test_invalid():
    symbol = "BDAD)DAAS&/9324423OUVibb"
    with pytest.raises(ValueError) as excinfo:
        res = pym.currency(symbol)
    assert "See 'symbols' or 'coins' properties" in str(excinfo)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pytest

from pymarketcap import AsyncPymarketcap
from pymarketcap.tests.consts import asyncparms

asyncparms["debug"] = False

@pytest.mark.end2end
def test_webs(event_loop):
    res = []
    async def get_all_currencies():
        async with AsyncPymarketcap(**asyncparms) as apym:
            async for currency in apym.every_currency():
                res.append(currency)

    event_loop.run_until_complete(get_all_currencies())

    web_found = False
    for r in res:
        print(r)
        if len(r["webs"]) > 0:
            web_found = True
    try:
        assert web_found == True
    except AssertionError as err:
        print("Any website found searching all currencies." \
        	  + "See 'processer:currency()' function.")
        raise err


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import choice

from pymarketcap.consts import EXCEPTIONAL_COIN_SLUGS_KEYS
from pymarketcap import Pymarketcap

pym = Pymarketcap()

def test_types():
    symbol = choice(pym.symbols)
    print("(Symbol: %s)" % symbol, end=" ")
    res = pym._is_symbol(symbol)
    assert type(res) == bool

    slug = choice(pym.coins)
    res = pym._is_symbol(slug)
    assert type(res) == bool

def test_consistence():
    # Symbol existent
    symbol = choice(pym.symbols)
    print("(Symbol: %s)" % symbol, end=" ")
    res = pym._is_symbol(symbol)
    assert res and symbol in pym.symbols

    # Eceptional symbol existent
    symbol = None
    for symbol in EXCEPTIONAL_COIN_SLUGS_KEYS:
        if symbol in pym.symbols:
            assert pym._is_symbol(symbol)
            break

    # Symbol not existent
    res = pym._is_symbol("OUBSAFYAIVADDOHNA")
    assert res  # It's True, see: processer._is_symbol.__doc__

    # Not symbol
    res = pym._is_symbol("ethereum")
    assert not res



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from random import choice
from pprint import pprint

from tqdm import tqdm

from pymarketcap import Pymarketcap
pym = Pymarketcap()

def random_coin():
    return choice(pym.coins)

def random_exchange():
    return choice(pym.exchange_slugs)

METHODS = [
    ["exchanges"],
    ["exchange", random_exchange()],
    ["currency", random_coin()],
    ["markets", random_coin()],
    ["ranks"],
    ["recently"],
    ["tokens"],
    ["graphs.currency", random_coin()],
    ["graphs.global_cap"],
    ["graphs.dominance"]
]

def test_fields():
    keys = []
    keys_method_map = {}
    for method in tqdm(METHODS, desc="Testing all response names consistency"):
        _exec = "pym.%s(%s)" % (method[0], '"%s"' % method[1] if len(method) == 2 else "")
        tqdm.write(_exec)
        try:
            res = eval(_exec)
        except IndexError:
            continue

        attempts = 5
        while attempts > 0:
            try:
                if method[0] == "ranks":
                    keys_from_method = list(res["gainers"]["1h"][0].keys())
                    keys.extend(keys_from_method)
                elif method[0] == "markets":
                    keys_from_method =list(res["markets"][0].keys())
                    keys.extend(keys_from_method)
                elif method[0] in ["currency", "stats",
                                   "graphs.currency",
                                   "graphs.global_cap",
                                   "graphs.dominance"]:
                    keys_from_method = list(res.keys())
                    keys.extend(keys_from_method)
                elif method[0] == "exchange":
                    keys_from_method = list(res.keys())
                    keys_from_method.extend(list(res["markets"][0].keys()))
                    keys.extend(keys_from_method)
                elif method[0] == "exchanges":
                    keys_from_method = list(res[0]["markets"][0].keys())
                    keys.extend(keys_from_method)
                elif method[0] in ["ticker", "tokens", "recently"]:
                    keys_from_method = list(res[0].keys())
                    keys.extend(keys_from_method)
                else:
                    tqdm.write("\nWARNING: Method %s() not tested" % method[0])
                    tqdm.write("RESPONSE: %r\n" % res)
            except Exception as err:
                tqdm.write(str(err))
                attempts -= 1
            else:
                for key in keys_from_method:
                    keys_method_map[key] = method[0]
                break

    # If you want to know from what method is every field,
    # insert it here and run the test:
    stops = [
        "website",
        "total_markets_cap",
        "total_markets_volume_24h"
    ]

    # Useful for maintainance
    #print(set(keys))

    for key in set(keys):
        if key in stops:
            msg = 'Key "%s" from method %s() not allowed.' % (key, keys_method_map[key])
            raise AssertionError(msg)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
from subprocess import check_call

import pytest

from pymarketcap import __title__

examples_folder = os.path.join(
	os.path.abspath(os.path.dirname(os.path.dirname(__file__))),
	"examples"
)

def assert_exec(f):
    filepath = os.path.join(examples_folder, f)
    print("Testing example %s" % filepath)
    ret_code = check_call([sys.executable, filepath])
    assert ret_code == 0

def test_basic_api_call():
    assert_exec("basic_api_call.py")

def test_basic_sync_scraper():
    assert_exec("basic_sync_scraper.py")

@pytest.mark.end2end
def test_basic_async_scraper():
    assert_exec("basic_async_scraper.py")

@pytest.mark.end2end
def test_fast_async_scraper():
    assert_exec("fast_async_scraper.py")




# External cython modules
from libc.stddef cimport size_t
from libc.string cimport memcpy
from cpython.mem cimport PyMem_Malloc, PyMem_Realloc, PyMem_Free
from curl cimport *

# Internal project modules
from pymarketcap.errors import CoinmarketcapHTTPError
from pymarketcap import __version__

# https://curl.haxx.se/libcurl/c/getinmemory.html
cdef struct MemoryStruct:
    char *memory
    size_t size

cdef size_t write_memory(void *contents, size_t size,
                         size_t nmemb, void *userp):
    cdef size_t realsize = size * nmemb
    cdef MemoryStruct *mem = <MemoryStruct *>userp
    mem.memory = <char *>PyMem_Realloc(
        mem.memory, mem.size + realsize + 1
    )
    if mem.memory == NULL:
        print("Not enough memory (realloc returned NULL)\n")
        return 0
    memcpy(&(mem.memory[mem.size]), contents, realsize)
    mem.size += realsize
    mem.memory[mem.size] = 0
    return realsize

cdef class Response(object):
    cdef readonly long status_code
    cdef readonly bytes text
    cdef readonly bytes content_type
    cdef readonly bytes encoding
    cdef readonly bytes url

    def __cinit__(self, text):
        self.text = text

cpdef Response get_to_memory(const char *url, long timeout,
                             bint debug):
    """Send a get request using a buffer stored in memory.

    Args:
        url (char *): A direction in memory of url.
        timeout (long): Number of seconds until expiration
            time cancels the request.
        debug (bint): Flag to activate/desactivate body
            response printing.

    Returns (Response):
        Returns a class with next attributes:
            ``text``, ``status_code``, 

    """
    cdef CURLcode ret
    cdef long true = 1L
    version = curl_version()
    cdef CURL *curl = curl_easy_init()
    cdef const char *user_agent = "pymarketcap 3.9.173"
    cdef const char *accept_encoding = "gzip, deflate"
    cdef char *raw_body

    cdef MemoryStruct chunk
    chunk.memory = <char *>PyMem_Malloc(1)
    chunk.size = 0

    if curl != NULL:
        if debug:
            curl_easy_setopt(curl, CURLOPT_VERBOSE,
                             &true)
        curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION,
                         &true)
        ret = curl_easy_setopt(curl, CURLOPT_URL, url)
        ret = curl_easy_setopt(curl, CURLOPT_TIMEOUT,
                               <void *>timeout)

        ret = curl_easy_setopt(curl, CURLOPT_HTTPGET,
                               &true)

        ret = curl_easy_setopt(curl, CURLOPT_USERAGENT,
                               user_agent)
        ret = curl_easy_setopt(curl, CURLOPT_ACCEPT_ENCODING,
                               accept_encoding)

        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,
                         &write_memory)
        curl_easy_setopt(curl, CURLOPT_WRITEDATA,
                         <void *>&chunk)

        if ret != CURLE_OK:
            raise RuntimeError
        ret = curl_easy_perform(curl)
        if ret != CURLE_OK:
            raise CoinmarketcapHTTPError

        resp = Response(chunk.memory)
        curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE,
                          &resp.status_code)
        curl_easy_cleanup(curl)

        PyMem_Free(chunk.memory)

        return resp



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap.tests import type_test
from pymarketcap import Pymarketcap
pym = Pymarketcap()

def assert_types(data):
    map_types = {
        "name":           str,
        "slug":           str,
        "symbol":         str,
        "volume_24h":     float,
        "price":          float,
        "percent_change": float,
    }
    assert isinstance(data, dict)
    for rank, rdata in data.items():
        assert isinstance(rank, str)
        assert isinstance(rdata, dict)
        for period, pdata in rdata.items():
            assert isinstance(period, str)
            assert isinstance(pdata, list)
            for currency in pdata:
                assert isinstance(currency, dict)
                for key, value in currency.items():
                    type_test(map_types, key, value)

def test_types():
    res = pym.ranks()
    assert_types(res)

def test_consistence():
    res = pym.ranks()

    # Check if a currency is repeated
    for rank, rdata in res.items():
        assert rank in ["gainers", "losers"]
        for period, pdata in rdata.items():
            assert period in ["1h", "24h", "7d"]




cdef extern from "curl/curl.h" nogil:
    ctypedef int CURLcode
    ctypedef void CURL
    ctypedef int CURLoption
    ctypedef int CURLINFO

    # {{{ CURLoption
    enum: CURLOPT_WRITEDATA
    enum: CURLOPT_URL
    enum: CURLOPT_PORT
    enum: CURLOPT_PROXY
    enum: CURLOPT_USERPWD
    enum: CURLOPT_PROXYUSERPWD
    enum: CURLOPT_RANGE
    enum: CURLOPT_READDATA
    enum: CURLOPT_ERRORBUFFER
    enum: CURLOPT_WRITEFUNCTION
    enum: CURLOPT_READFUNCTION
    enum: CURLOPT_TIMEOUT
    enum: CURLOPT_INFILESIZE
    enum: CURLOPT_POSTFIELDS
    enum: CURLOPT_REFERER
    enum: CURLOPT_FTPPORT
    enum: CURLOPT_USERAGENT
    enum: CURLOPT_LOW_SPEED_LIMIT
    enum: CURLOPT_LOW_SPEED_TIME
    enum: CURLOPT_RESUME_FROM
    enum: CURLOPT_COOKIE
    enum: CURLOPT_HTTPHEADER
    enum: CURLOPT_HTTPPOST
    enum: CURLOPT_SSLCERT
    enum: CURLOPT_KEYPASSWD
    enum: CURLOPT_CRLF
    enum: CURLOPT_QUOTE
    enum: CURLOPT_HEADERDATA
    enum: CURLOPT_COOKIEFILE
    enum: CURLOPT_SSLVERSION
    enum: CURLOPT_TIMECONDITION
    enum: CURLOPT_TIMEVALUE
    enum: CURLOPT_CUSTOMREQUEST
    enum: CURLOPT_STDERR
    enum: CURLOPT_POSTQUOTE
    enum: CURLOPT_OBSOLETE40
    enum: CURLOPT_VERBOSE
    enum: CURLOPT_HEADER
    enum: CURLOPT_NOPROGRESS
    enum: CURLOPT_NOBODY
    enum: CURLOPT_FAILONERROR
    enum: CURLOPT_UPLOAD
    enum: CURLOPT_POST
    enum: CURLOPT_DIRLISTONLY
    enum: CURLOPT_APPEND
    enum: CURLOPT_NETRC
    enum: CURLOPT_FOLLOWLOCATION
    enum: CURLOPT_TRANSFERTEXT
    enum: CURLOPT_PUT
    enum: CURLOPT_PROGRESSFUNCTION
    enum: CURLOPT_PROGRESSDATA
    enum: CURLOPT_AUTOREFERER
    enum: CURLOPT_PROXYPORT
    enum: CURLOPT_POSTFIELDSIZE
    enum: CURLOPT_HTTPPROXYTUNNEL
    enum: CURLOPT_INTERFACE
    enum: CURLOPT_KRBLEVEL
    enum: CURLOPT_SSL_VERIFYPEER
    enum: CURLOPT_CAINFO
    enum: CURLOPT_MAXREDIRS
    enum: CURLOPT_FILETIME
    enum: CURLOPT_TELNETOPTIONS
    enum: CURLOPT_MAXCONNECTS
    enum: CURLOPT_OBSOLETE72
    enum: CURLOPT_FRESH_CONNECT
    enum: CURLOPT_FORBID_REUSE
    enum: CURLOPT_RANDOM_FILE
    enum: CURLOPT_EGDSOCKET
    enum: CURLOPT_CONNECTTIMEOUT
    enum: CURLOPT_HEADERFUNCTION
    enum: CURLOPT_HTTPGET
    enum: CURLOPT_SSL_VERIFYHOST
    enum: CURLOPT_COOKIEJAR
    enum: CURLOPT_SSL_CIPHER_LIST
    enum: CURLOPT_HTTP_VERSION
    enum: CURLOPT_FTP_USE_EPSV
    enum: CURLOPT_SSLCERTTYPE
    enum: CURLOPT_SSLKEY
    enum: CURLOPT_SSLKEYTYPE
    enum: CURLOPT_SSLENGINE
    enum: CURLOPT_SSLENGINE_DEFAULT
    enum: CURLOPT_DNS_USE_GLOBAL_CACHE
    enum: CURLOPT_DNS_CACHE_TIMEOUT
    enum: CURLOPT_PREQUOTE
    enum: CURLOPT_DEBUGFUNCTION
    enum: CURLOPT_DEBUGDATA
    enum: CURLOPT_COOKIESESSION
    enum: CURLOPT_CAPATH
    enum: CURLOPT_BUFFERSIZE
    enum: CURLOPT_NOSIGNAL
    enum: CURLOPT_SHARE
    enum: CURLOPT_PROXYTYPE
    enum: CURLOPT_ACCEPT_ENCODING
    enum: CURLOPT_PRIVATE
    enum: CURLOPT_HTTP200ALIASES
    enum: CURLOPT_UNRESTRICTED_AUTH
    enum: CURLOPT_FTP_USE_EPRT
    enum: CURLOPT_HTTPAUTH
    enum: CURLOPT_SSL_CTX_FUNCTION
    enum: CURLOPT_SSL_CTX_DATA
    enum: CURLOPT_FTP_CREATE_MISSING_DIRS
    enum: CURLOPT_PROXYAUTH
    enum: CURLOPT_FTP_RESPONSE_TIMEOUT
    enum: CURLOPT_IPRESOLVE
    enum: CURLOPT_MAXFILESIZE
    enum: CURLOPT_INFILESIZE_LARGE
    enum: CURLOPT_RESUME_FROM_LARGE
    enum: CURLOPT_MAXFILESIZE_LARGE
    enum: CURLOPT_NETRC_FILE
    enum: CURLOPT_USE_SSL
    enum: CURLOPT_POSTFIELDSIZE_LARGE
    enum: CURLOPT_TCP_NODELAY
    enum: CURLOPT_FTPSSLAUTH
    enum: CURLOPT_IOCTLFUNCTION
    enum: CURLOPT_IOCTLDATA
    enum: CURLOPT_FTP_ACCOUNT
    enum: CURLOPT_COOKIELIST
    enum: CURLOPT_IGNORE_CONTENT_LENGTH
    enum: CURLOPT_FTP_SKIP_PASV_IP
    enum: CURLOPT_FTP_FILEMETHOD
    enum: CURLOPT_LOCALPORT
    enum: CURLOPT_LOCALPORTRANGE
    enum: CURLOPT_CONNECT_ONLY
    enum: CURLOPT_CONV_FROM_NETWORK_FUNCTION
    enum: CURLOPT_CONV_TO_NETWORK_FUNCTION
    enum: CURLOPT_CONV_FROM_UTF8_FUNCTION
    enum: CURLOPT_MAX_SEND_SPEED_LARGE
    enum: CURLOPT_MAX_RECV_SPEED_LARGE
    enum: CURLOPT_FTP_ALTERNATIVE_TO_USER
    enum: CURLOPT_SOCKOPTFUNCTION
    enum: CURLOPT_SOCKOPTDATA
    enum: CURLOPT_SSL_SESSIONID_CACHE
    enum: CURLOPT_SSH_AUTH_TYPES
    enum: CURLOPT_SSH_PUBLIC_KEYFILE
    enum: CURLOPT_SSH_PRIVATE_KEYFILE
    enum: CURLOPT_FTP_SSL_CCC
    enum: CURLOPT_TIMEOUT_MS
    enum: CURLOPT_CONNECTTIMEOUT_MS
    enum: CURLOPT_HTTP_TRANSFER_DECODING
    enum: CURLOPT_HTTP_CONTENT_DECODING
    enum: CURLOPT_NEW_FILE_PERMS
    enum: CURLOPT_NEW_DIRECTORY_PERMS
    enum: CURLOPT_POSTREDIR
    enum: CURLOPT_SSH_HOST_PUBLIC_KEY_MD5
    enum: CURLOPT_OPENSOCKETFUNCTION
    enum: CURLOPT_OPENSOCKETDATA
    enum: CURLOPT_COPYPOSTFIELDS
    enum: CURLOPT_PROXY_TRANSFER_MODE
    enum: CURLOPT_SEEKFUNCTION
    enum: CURLOPT_SEEKDATA
    enum: CURLOPT_CRLFILE
    enum: CURLOPT_ISSUERCERT
    enum: CURLOPT_ADDRESS_SCOPE
    enum: CURLOPT_CERTINFO
    enum: CURLOPT_USERNAME
    enum: CURLOPT_PASSWORD
    enum: CURLOPT_PROXYUSERNAME
    enum: CURLOPT_PROXYPASSWORD
    enum: CURLOPT_NOPROXY
    enum: CURLOPT_TFTP_BLKSIZE
    enum: CURLOPT_SOCKS5_GSSAPI_SERVICE
    enum: CURLOPT_SOCKS5_GSSAPI_NEC
    enum: CURLOPT_PROTOCOLS
    enum: CURLOPT_REDIR_PROTOCOLS
    enum: CURLOPT_SSH_KNOWNHOSTS
    enum: CURLOPT_SSH_KEYFUNCTION
    enum: CURLOPT_SSH_KEYDATA
    enum: CURLOPT_MAIL_FROM
    enum: CURLOPT_MAIL_RCPT
    enum: CURLOPT_FTP_USE_PRET
    enum: CURLOPT_RTSP_REQUEST
    enum: CURLOPT_RTSP_SESSION_ID
    enum: CURLOPT_RTSP_STREAM_URI
    enum: CURLOPT_RTSP_TRANSPORT
    enum: CURLOPT_RTSP_CLIENT_CSEQ
    enum: CURLOPT_RTSP_SERVER_CSEQ
    enum: CURLOPT_INTERLEAVEDATA
    enum: CURLOPT_INTERLEAVEFUNCTION
    enum: CURLOPT_WILDCARDMATCH
    enum: CURLOPT_CHUNK_BGN_FUNCTION
    enum: CURLOPT_CHUNK_END_FUNCTION
    enum: CURLOPT_FNMATCH_FUNCTION
    enum: CURLOPT_CHUNK_DATA
    enum: CURLOPT_FNMATCH_DATA
    enum: CURLOPT_RESOLVE
    enum: CURLOPT_TLSAUTH_USERNAME
    enum: CURLOPT_TLSAUTH_PASSWORD
    enum: CURLOPT_TLSAUTH_TYPE
    enum: CURLOPT_TRANSFER_ENCODING
    enum: CURLOPT_CLOSESOCKETFUNCTION
    enum: CURLOPT_CLOSESOCKETDATA
    enum: CURLOPT_GSSAPI_DELEGATION
    enum: CURLOPT_DNS_SERVERS
    enum: CURLOPT_ACCEPTTIMEOUT_MS
    enum: CURLOPT_TCP_KEEPALIVE
    enum: CURLOPT_TCP_KEEPIDLE
    enum: CURLOPT_TCP_KEEPINTVL
    enum: CURLOPT_SSL_OPTIONS
    enum: CURLOPT_MAIL_AUTH
    enum: CURLOPT_SASL_IR
    enum: CURLOPT_XFERINFOFUNCTION
    enum: CURLOPT_XOAUTH2_BEARER
    enum: CURLOPT_DNS_INTERFACE
    enum: CURLOPT_DNS_LOCAL_IP4
    enum: CURLOPT_DNS_LOCAL_IP6
    enum: CURLOPT_LOGIN_OPTIONS
    enum: CURLOPT_SSL_ENABLE_NPN
    enum: CURLOPT_SSL_ENABLE_ALPN
    enum: CURLOPT_EXPECT_100_TIMEOUT_MS
    enum: CURLOPT_PROXYHEADER
    enum: CURLOPT_HEADEROPT
    # }}}

    # {{{ CURLcode
    enum: CURLE_OK
    enum: CURLE_UNSUPPORTED_PROTOCOL
    enum: CURLE_FAILED_INIT
    enum: CURLE_URL_MALFORMAT
    enum: CURLE_NOT_BUILT_IN
    enum: CURLE_COULDNT_RESOLVE_PROXY
    enum: CURLE_COULDNT_RESOLVE_HOST
    enum: CURLE_COULDNT_CONNECT
    enum: CURLE_FTP_WEIRD_SERVER_REPLY
    enum: CURLE_REMOTE_ACCESS_DENIED
    enum: CURLE_FTP_ACCEPT_FAILED
    enum: CURLE_FTP_WEIRD_PASS_REPLY
    enum: CURLE_FTP_ACCEPT_TIMEOUT
    enum: CURLE_FTP_WEIRD_PASV_REPLY
    enum: CURLE_FTP_WEIRD_227_FORMAT
    enum: CURLE_FTP_CANT_GET_HOST
    enum: CURLE_HTTP2
    enum: CURLE_FTP_COULDNT_SET_TYPE
    enum: CURLE_PARTIAL_FILE
    enum: CURLE_FTP_COULDNT_RETR_FILE
    enum: CURLE_OBSOLETE20
    enum: CURLE_QUOTE_ERROR
    enum: CURLE_HTTP_RETURNED_ERROR
    enum: CURLE_WRITE_ERROR
    enum: CURLE_OBSOLETE24
    enum: CURLE_UPLOAD_FAILED
    enum: CURLE_READ_ERROR
    enum: CURLE_OUT_OF_MEMORY
    enum: CURLE_OPERATION_TIMEDOUT
    enum: CURLE_OBSOLETE29
    enum: CURLE_FTP_PORT_FAILED
    enum: CURLE_FTP_COULDNT_USE_REST
    enum: CURLE_OBSOLETE32
    enum: CURLE_RANGE_ERROR
    enum: CURLE_HTTP_POST_ERROR
    enum: CURLE_SSL_CONNECT_ERROR
    enum: CURLE_BAD_DOWNLOAD_RESUME
    enum: CURLE_FILE_COULDNT_READ_FILE
    enum: CURLE_LDAP_CANNOT_BIND
    enum: CURLE_LDAP_SEARCH_FAILED
    enum: CURLE_OBSOLETE40
    enum: CURLE_FUNCTION_NOT_FOUND
    enum: CURLE_ABORTED_BY_CALLBACK
    enum: CURLE_BAD_FUNCTION_ARGUMENT
    enum: CURLE_OBSOLETE44
    enum: CURLE_INTERFACE_FAILED
    enum: CURLE_OBSOLETE46
    enum: CURLE_TOO_MANY_REDIRECTS
    enum: CURLE_UNKNOWN_OPTION
    enum: CURLE_TELNET_OPTION_SYNTAX
    enum: CURLE_OBSOLETE50
    enum: CURLE_PEER_FAILED_VERIFICATION
    enum: CURLE_GOT_NOTHING
    enum: CURLE_SSL_ENGINE_NOTFOUND
    enum: CURLE_SSL_ENGINE_SETFAILED
    enum: CURLE_SEND_ERROR
    enum: CURLE_RECV_ERROR
    enum: CURLE_OBSOLETE57
    enum: CURLE_SSL_CERTPROBLEM
    enum: CURLE_SSL_CIPHER
    enum: CURLE_SSL_CACERT
    enum: CURLE_BAD_CONTENT_ENCODING
    enum: CURLE_LDAP_INVALID_URL
    enum: CURLE_FILESIZE_EXCEEDED
    enum: CURLE_USE_SSL_FAILED
    enum: CURLE_SEND_FAIL_REWIND
    enum: CURLE_SSL_ENGINE_INITFAILED
    enum: CURLE_LOGIN_DENIED
    enum: CURLE_TFTP_NOTFOUND
    enum: CURLE_TFTP_PERM
    enum: CURLE_REMOTE_DISK_FULL
    enum: CURLE_TFTP_ILLEGAL
    enum: CURLE_TFTP_UNKNOWNID
    enum: CURLE_REMOTE_FILE_EXISTS
    enum: CURLE_TFTP_NOSUCHUSER
    enum: CURLE_CONV_FAILED
    enum: CURLE_CONV_REQD
    enum: CURLE_SSL_CACERT_BADFILE
    enum: CURLE_REMOTE_FILE_NOT_FOUND
    enum: CURLE_SSH
    enum: CURLE_SSL_SHUTDOWN_FAILED
    enum: CURLE_AGAIN
    enum: CURLE_SSL_CRL_BADFILE
    enum: CURLE_SSL_ISSUER_ERROR
    enum: CURLE_FTP_PRET_FAILED
    enum: CURLE_RTSP_CSEQ_ERROR
    enum: CURLE_RTSP_SESSION_ERROR
    enum: CURLE_FTP_BAD_FILE_LIST
    enum: CURLE_CHUNK_FAILED
    enum: CURLE_NO_CONNECTION_AVAILABLE
    enum: CURL_LAST
    # }}}

    # {{{ CURLINFO
    enum: CURLINFO_NONE
    enum: CURLINFO_EFFECTIVE_URL
    enum: CURLINFO_RESPONSE_CODE
    enum: CURLINFO_TOTAL_TIME
    enum: CURLINFO_NAMELOOKUP_TIME
    enum: CURLINFO_CONNECT_TIME
    enum: CURLINFO_PRETRANSFER_TIME
    enum: CURLINFO_SIZE_UPLOAD
    enum: CURLINFO_SIZE_DOWNLOAD
    enum: CURLINFO_SPEED_DOWNLOAD
    enum: CURLINFO_SPEED_UPLOAD
    enum: CURLINFO_HEADER_SIZE
    enum: CURLINFO_REQUEST_SIZE
    enum: CURLINFO_SSL_VERIFYRESULT
    enum: CURLINFO_FILETIME
    enum: CURLINFO_CONTENT_LENGTH_DOWNLOAD
    enum: CURLINFO_CONTENT_LENGTH_UPLOAD
    enum: CURLINFO_STARTTRANSFER_TIME
    enum: CURLINFO_CONTENT_TYPE
    enum: CURLINFO_REDIRECT_TIME
    enum: CURLINFO_REDIRECT_COUNT
    enum: CURLINFO_PRIVATE
    enum: CURLINFO_HTTP_CONNECTCODE
    enum: CURLINFO_HTTPAUTH_AVAIL
    enum: CURLINFO_PROXYAUTH_AVAIL
    enum: CURLINFO_OS_ERRNO
    enum: CURLINFO_NUM_CONNECTS
    enum: CURLINFO_SSL_ENGINES
    enum: CURLINFO_COOKIELIST
    enum: CURLINFO_LASTSOCKET
    enum: CURLINFO_FTP_ENTRY_PATH
    enum: CURLINFO_REDIRECT_URL
    enum: CURLINFO_PRIMARY_IP
    enum: CURLINFO_APPCONNECT_TIME
    enum: CURLINFO_CERTINFO
    enum: CURLINFO_CONDITION_UNMET
    enum: CURLINFO_RTSP_SESSION_ID
    enum: CURLINFO_RTSP_CLIENT_CSEQ
    enum: CURLINFO_RTSP_SERVER_CSEQ
    enum: CURLINFO_RTSP_CSEQ_RECV
    enum: CURLINFO_PRIMARY_PORT
    enum: CURLINFO_LOCAL_IP
    enum: CURLINFO_LOCAL_PORT
    enum: CURLINFO_TLS_SESSION
    enum: CURLINFO_LASTONE
    # }}}

    char *curl_version()
    CURL *curl_easy_init()
    CURLcode curl_easy_perform(CURL * easy_handle )
    void curl_easy_cleanup(CURL * handle )
    CURLcode curl_easy_setopt(CURL *handle, CURLoption option, void *parameter)
    const char *curl_easy_strerror(CURLcode errornum)
    CURLcode curl_easy_getinfo(CURL *curl, CURLINFO info, ... )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tests for stats() method"""

from time import sleep
from random import choice

from pymarketcap.tests import type_test
from pymarketcap import Pymarketcap
pym = Pymarketcap()

all_badges = list(pym.ticker_badges)
all_badges.remove("USD")

def teardown_function(function):
    sleep(1)

def assert_types(res):
    int_type_fields = [
        "active_assets",
        "active_currencies",
        "active_markets",
        "last_updated",
    ]

    assert isinstance(res, dict)
    for key, value in res.items():
        if key in int_type_fields:
            assert isinstance(value, int)
        else:
            assert isinstance(value, float)

def test_types():
    res = pym.stats()
    assert_types(res)

def assert_consistence(res, with_convert=False):
    if with_convert:
        assert len(res) == 9
    else:
        assert len(res) == 7

def test_consistence():
    res = pym.stats()
    assert_consistence(res)

def test_convert():
    badge = choice(all_badges)
    res = pym.stats(convert=badge)

    keys_with_badge = 0
    for key in res:
        if badge.lower() in key:
            keys_with_badge += 1
    assert keys_with_badge == 2

    assert_types(res)
    assert_consistence(res, with_convert=True)



# -*- coding: utf-8 -*-

"""Pymarketcap source code."""

import sys

__title__ = "pymarketcap"
__version__ = "3.9.173"
__version_info__ = (int(num) for num in __version__.split("."))
__author__ = "Álvaro Mondéjar Rubio <mondejar1994@gmail.com>"
__repo__ = "https://github.com/mondeja/pymarketcap"
__license__ = "BSD License"

from pymarketcap.core import Pymarketcap
from pymarketcap.errors import (
    CoinmarketcapError,
    CoinmarketcapHTTPError,
    CoinmarketcapTooManyRequestsError
)
if sys.version_info >= (3, 6):
    from pymarketcap.pymasyncore import AsyncPymarketcap



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap

def main():
    cmc = Pymarketcap()

    # Get all currencies ranked by volume
    currencies = cmc.ticker()
    print(currencies[0])  # Show first currency

if __name__ == "__main__":
    main()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from pymarketcap import Pymarketcap
pym = Pymarketcap()

def test_types():
    res = pym._cache_exchanges_ids()

    assert isinstance(res, list)
    for exc in res:
        assert isinstance(exc, list)
        assert isinstance(exc[0], str)
        assert isinstance(exc[1], str)
        assert isinstance(exc[2], int)

def test_consistence():
    res = pym._cache_exchanges_ids()

    assert len(res) > 0

    names = []
    slugs = []
    ids = []
    for exc in res:
        assert len(exc) == 3
        _name, _slug, _id = (exc[0], exc[1], exc[2])
        assert _name not in names
        assert _slug not in slugs
        assert _id not in ids

        names.append(_name)
        slugs.append(_slug)
        ids.append(_id)






# -*- coding: utf-8 -*-

"""Urllib implementation for pymarketcap."""

from urllib.error import HTTPError
from urllib.request import urlopen, Request
from socket import timeout as TimeoutHTTPError

from pymarketcap import __version__
from pymarketcap.errors import CoinmarketcapHTTPError408

class Response:
    """Internal response object for encapsulate responses
    getted by requests with urllib module."""
    def __init__(self, text, status_code, url):
        self.text = text
        self.status_code = status_code
        self.url = url

def get_to_memory(url, timeout, debug):
    """GET request stored in memory.

    Args:
        url: Url to send GET request.
        timeout (int): Number of seconds until
            expiration time cancels the request.
        debug (bool): See code response or not.
    """
    req = Request(
        url.decode(),
        headers={"User-Agent": "pymarketcap %s" % __version__}
    )
    try:
        req = urlopen(req, timeout=timeout)
    except HTTPError as err:
        return Response(b"", err.code, url)
    except OSError:
        return Response(b"", 404, url)
    except TimeoutHTTPError:
        raise CoinmarketcapHTTPError408(
            "Request timeout exceed (%d seconds)" % timeout
        )
    else:
        data = req.read()
        if debug:
            print(data)
        res = Response(data, req.getcode(), url)
        req.close()
        return res
